@@ -1,6 +1,4 @@
-use std::collections::HashMap;
 use std::fmt::Debug;
-use std::fmt::Display;
 use std::io::Write;
 use std::usize;
 
@@ -27,22 +25,13 @@ impl Debug for XMLError {
     }
 }
 
-impl Display for XML {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let mut s: Vec<u8> = Vec::new();
-        self.render(&mut s)
-            .expect("Failure writing output to Vec<u8>");
-
-        write!(f, "{}", unsafe { String::from_utf8_unchecked(s) })
-    }
-}
-
 impl Default for XML {
     fn default() -> Self {
         XML {
             version: "1.0".into(),
             encoding: "UTF-8".into(),
             should_render_header: true,
+            should_sort_attributes: false,
             custom_header: None,
             root: None,
         }
@@ -55,6 +44,7 @@ pub struct XML {
     version: String,
     encoding: String,
     should_render_header: bool,
+    should_sort_attributes: bool,
     custom_header: Option<String>,
     root: Option<XMLElement>,
 }
@@ -82,6 +72,11 @@ impl XML {
         self.encoding = encoding;
     }
 
+    /// Sets the header attribute sort.
+    pub fn set_attribute_sorting(&mut self, should_sort: bool) {
+        self.should_sort_attributes = should_sort;
+    }
+
     /// Sets a custom XML header.
     ///
     /// Be careful, no syntax and semantic verifications are made on this header.
@@ -94,10 +89,10 @@ impl XML {
         self.root = Some(element);
     }
 
-    /// Renders an XML document into the specified writer implementing Write trait.
+    /// Builds an XML document into the specified writer implementing Write trait.
     ///
-    /// Does not take ownership of the object.
-    pub fn render<W: Write>(&self, mut writer: W) -> Result<()> {
+    /// Consumes the XML object.
+    pub fn build<W: Write>(self, mut writer: W) -> Result<()> {
         // Rendering first XML header line if asked...
         if self.should_render_header {
             if let Some(header) = &self.custom_header {
@@ -105,32 +100,26 @@ impl XML {
             } else {
                 writeln!(
                     writer,
-                    r#"<?xml version="{}" encoding="{}"?>"#,
-                    self.version, self.encoding
+                    r#"<?xml encoding="{}" version="{}"?>"#,
+                    self.encoding, self.version
                 )?;
             }
         }
 
         // And then XML elements if present...
         if let Some(elem) = &self.root {
-            elem.render(&mut writer)?;
+            elem.render(&mut writer, self.should_sort_attributes)?;
         }
 
         Ok(())
     }
-
-    /// Builds an XML document into the specified writer implementing Write trait.
-    ///
-    /// Consumes the XML object.
-    pub fn build<W: Write>(self, writer: W) -> Result<()> {
-        self.render(writer)
-    }
 }
 
 /// Structure representing an XML element field.
 pub struct XMLElement {
     name: String,
-    attributes: HashMap<String, String>,
+    attributes: Vec<(String, String)>,
+    should_sort_attributes: Option<bool>,
     content: XMLElementContent,
 }
 
@@ -142,12 +131,17 @@ impl XMLElement {
     /// * `name` - A string slice that holds the name of the XML element.
     pub fn new(name: &str) -> Self {
         XMLElement {
-            name: name.to_owned(),
-            attributes: HashMap::new(),
+            name: name.into(),
+            attributes: Vec::new(),
+            should_sort_attributes: None,
             content: XMLElementContent::Empty,
         }
     }
 
+    pub fn set_attribute_sorting(&mut self, should_sort: bool) {
+        self.should_sort_attributes = Some(should_sort);
+    }
+
     /// Adds the given name/value attribute to the XMLElement.
     ///
     /// # Arguments
@@ -155,7 +149,7 @@ impl XMLElement {
     /// * `name` - A string slice that holds the name of the attribute
     /// * `value` - A string slice that holds the value of the attribute
     pub fn add_attribute(&mut self, name: &str, value: &str) {
-        self.attributes.insert(name.to_owned(), escape_str(value));
+        self.attributes.push((name.into(), escape_str(value)));
     }
 
     /// Adds a new XMLElement child object to the references XMLElement.
@@ -204,13 +198,23 @@ impl XMLElement {
     }
 
     /// Internal method rendering an attribute list to a String.
-    fn attributes_as_string(&self) -> String {
+    fn attributes_as_string(&self, should_sort: bool) -> String {
         if self.attributes.is_empty() {
             "".to_owned()
         } else {
-            let mut result = "".to_owned();
-            for (k, v) in &self.attributes {
-                result = result + &format!(r#" {}="{}""#, k, v);
+            let mut attributes = self.attributes.clone();
+
+            // Giving priority to the element boolean, and taking the global xml if not set
+            let should_sort_attributes = self.should_sort_attributes.unwrap_or(should_sort);
+
+            if should_sort_attributes {
+                attributes.sort();
+            }
+
+            let mut result = "".into();
+
+            for (k, v) in &attributes {
+                result = format!("{} {}", result, format!(r#"{}="{}""#, k, v));
             }
             result
         }
@@ -222,8 +226,8 @@ impl XMLElement {
     /// # Arguments
     ///
     /// * `writer` - An object to render the referenced XMLElement to
-    pub fn render<W: Write>(&self, writer: &mut W) -> Result<()> {
-        self.render_level(writer, 0)
+    pub fn render<W: Write>(&self, writer: &mut W, should_sort: bool) -> Result<()> {
+        self.render_level(writer, 0, should_sort)
     }
 
     /// Internal method rendering and indenting a XMLELement object
@@ -232,29 +236,23 @@ impl XMLElement {
     ///
     /// * `writer` - An object to render the referenced XMLElement to
     /// * `level` - An usize representing the depth of the XML tree. Used to indent the object.
-    fn render_level<W: Write>(&self, writer: &mut W, level: usize) -> Result<()> {
+    fn render_level<W: Write>(
+        &self,
+        writer: &mut W,
+        level: usize,
+        should_sort: bool,
+    ) -> Result<()> {
         let indent = "\t".repeat(level);
+        let attributes = self.attributes_as_string(should_sort);
 
         match &self.content {
             XMLElementContent::Empty => {
-                writeln!(
-                    writer,
-                    "{}<{}{} />",
-                    indent,
-                    self.name,
-                    self.attributes_as_string()
-                )?;
+                writeln!(writer, "{}<{}{} />", indent, self.name, attributes)?;
             }
             XMLElementContent::Elements(elements) => {
-                writeln!(
-                    writer,
-                    "{}<{}{}>",
-                    indent,
-                    self.name,
-                    self.attributes_as_string()
-                )?;
+                writeln!(writer, "{}<{}{}>", indent, self.name, attributes)?;
                 for elem in elements {
-                    elem.render_level(writer, level + 1)?;
+                    elem.render_level(writer, level + 1, should_sort)?;
                 }
                 writeln!(writer, "{}</{}>", indent, self.name)?;
             }
@@ -262,11 +260,7 @@ impl XMLElement {
                 writeln!(
                     writer,
                     "{}<{}{}>{}</{}>",
-                    indent,
-                    self.name,
-                    self.attributes_as_string(),
-                    text,
-                    self.name
+                    indent, self.name, attributes, text, self.name
                 )?;
             }
         };
@@ -294,7 +288,6 @@ enum XMLElementContent {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use std::io::Cursor;
 
     #[test]
     fn test_xml_default_creation() {
@@ -307,10 +300,8 @@ mod tests {
     fn test_xml_file_write() {
         let xml = XML::new();
 
-        let vec: Vec<u8> = Vec::new();
-        let writer = Cursor::new(vec);
-
-        xml.build(writer).unwrap();
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
     }
 
     #[test]
@@ -318,13 +309,13 @@ mod tests {
         let mut xml = XML::new();
         xml.set_version("1.1".into());
 
-        let expected = "<?xml version=\"1.1\" encoding=\"UTF-8\"?>\n";
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
 
-        assert_eq!(
-            format!("{}", xml),
-            expected,
-            "Both values does not match..."
-        );
+        let expected = "<?xml encoding=\"UTF-8\" version=\"1.1\"?>\n";
+        let res = std::str::from_utf8(&writer).unwrap();
+
+        assert_eq!(res, expected, "Both values does not match...");
     }
 
     #[test]
@@ -332,13 +323,13 @@ mod tests {
         let mut xml = XML::new();
         xml.set_encoding("UTF-16".into());
 
-        let expected = "<?xml version=\"1.0\" encoding=\"UTF-16\"?>\n";
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
 
-        assert_eq!(
-            format!("{}", xml),
-            expected,
-            "Both values does not match..."
-        );
+        let expected = "<?xml encoding=\"UTF-16\" version=\"1.0\"?>\n";
+        let res = std::str::from_utf8(&writer).unwrap();
+
+        assert_eq!(res, expected, "Both values does not match...");
     }
 
     #[test]
@@ -348,13 +339,13 @@ mod tests {
 
         xml.set_custom_header(header);
 
-        let expected = "<tv generator-info-name=\"my listings generator\">\n";
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
 
-        assert_eq!(
-            format!("{}", xml),
-            expected,
-            "Both values does not match..."
-        );
+        let expected = "<tv generator-info-name=\"my listings generator\">\n";
+        let res = std::str::from_utf8(&writer).unwrap();
+
+        assert_eq!(res, expected, "Both values does not match...");
     }
 
     #[test]
@@ -376,17 +367,92 @@ mod tests {
 
         xml.set_root_element(house);
 
-        let expected = "<?xml version=\"1.1\" encoding=\"UTF-8\"?>
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
+
+        let expected = "<?xml encoding=\"UTF-8\" version=\"1.1\"?>
 <house rooms=\"2\">
 \t<room number=\"1\">This is room number 1</room>
 \t<room number=\"2\">This is room number 2</room>
 </house>\n";
+        let res = std::str::from_utf8(&writer).unwrap();
 
-        assert_eq!(
-            format!("{}", xml),
-            expected,
-            "Both values does not match..."
-        )
+        assert_eq!(res, expected, "Both values does not match...")
+    }
+
+    // Here the `sort` attribute is set to the root, so everything should be sorted
+    #[test]
+    fn test_complex_sorted_root_xml() {
+        let mut xml = XML::new();
+        xml.set_attribute_sorting(true);
+        xml.set_version("1.1".into());
+        xml.set_encoding("UTF-8".into());
+
+        let mut house = XMLElement::new("house");
+        house.add_attribute("rooms", "2");
+
+        for i in 1..=2 {
+            let mut room = XMLElement::new("room");
+            room.add_attribute("size", &(i * 27).to_string());
+            room.add_attribute("number", &i.to_string());
+            room.add_text(format!("This is room number {}", i)).unwrap();
+
+            house.add_child(room).unwrap();
+        }
+
+        xml.set_root_element(house);
+
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
+
+        let expected = "<?xml encoding=\"UTF-8\" version=\"1.1\"?>
+<house rooms=\"2\">
+\t<room number=\"1\" size=\"27\">This is room number 1</room>
+\t<room number=\"2\" size=\"54\">This is room number 2</room>
+</house>\n";
+        let res = std::str::from_utf8(&writer).unwrap();
+
+        assert_eq!(res, expected, "Both values does not match...")
+    }
+
+    // Here the `sort` attribute is set to the an element only, so everything should not be sorted
+    #[test]
+    fn test_complex_sorted_element_xml() {
+        let mut xml = XML::new();
+        xml.set_version("1.1".into());
+        xml.set_encoding("UTF-8".into());
+
+        let mut house = XMLElement::new("house");
+        house.add_attribute("rooms", "2");
+
+        for i in 1..=2 {
+            let mut room = XMLElement::new("room");
+            room.add_attribute("size", &(i * 27).to_string());
+            room.add_attribute("city", ["Paris", "LA"][i - 1]);
+            room.add_attribute("number", &i.to_string());
+            room.add_text(format!("This is room number {}", i)).unwrap();
+
+            if i % 2 == 0 {
+                room.set_attribute_sorting(true);
+            }
+
+            house.add_child(room).unwrap();
+        }
+
+        xml.set_root_element(house);
+
+        let mut writer: Vec<u8> = Vec::new();
+        xml.build(&mut writer).unwrap();
+
+        let expected = "<?xml encoding=\"UTF-8\" version=\"1.1\"?>
+<house rooms=\"2\">
+\t<room size=\"27\" city=\"Paris\" number=\"1\">This is room number 1</room>
+\t<room city=\"LA\" number=\"2\" size=\"54\">This is room number 2</room>
+</house>\n";
+
+        let res = std::str::from_utf8(&writer).unwrap();
+
+        assert_eq!(res, expected, "Both values does not match...")
     }
 
     #[test]
@@ -402,6 +468,6 @@ mod tests {
         let xml_child2 = XMLElement::new("sorry");
         xml_child.add_child(xml_child2).unwrap();
 
-        xml.render(std::io::stdout()).unwrap();
+        xml.build(std::io::stdout()).unwrap();
     }
 }
